import click
import sys
import json
import datetime
from pygments import highlight
from pygments.formatters import TerminalFormatter
from pygments.lexers import JsonLexer


def validate_duration(ctx, param, value):
    try:
        if value is None:
            return None
        from pytimeparse import parse

        seconds = parse(value)
        if seconds <= 0:
            raise click.BadParameter("Duration should be greater than 0")
        return datetime.timedelta(seconds=seconds)
    except ValueError:
        raise click.BadParameter("Invalid duration (try '24h' or '7 days'")


def validate_datetime(ctx, param, value):
    try:
        from dateutil.parser import parse

        return parse(value)
    except ValueError:
        raise click.BadParameter("Invalid date/time")


CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])


@click.command(context_settings=CONTEXT_SETTINGS)
@click.option("--consumer_key", help="Consumer key", required=True)
@click.option("--consumer_secret", help="Consumer secret", required=True)
@click.option("--access_token_key", help="Access token key", required=True)
@click.option("--access_token_secret", help="Access token secret", required=True)
@click.option(
    "--delete_older_than",
    help="Delete all tweets older than this duration",
    default="7 days",
    callback=validate_duration,
)
@click.option(
    "--delete_everything_after",
    default="March 21, 2006",
    help="Only delete tweets that were created after this date",
    callback=validate_datetime,
)
@click.option(
    "--minimum_engagement",
    type=int,
    help="Minimum engagement count. ♥️  = 1, ♻️  = 2. Tweets below this amount are deleted. Set to a very high number to delete everything.",
    required=True,
)
@click.option(
    "--remove_favorites",
    help="Optionally remove all favorites, filtered by --delete_older_than (if specified)",
    is_flag=True,
)
def cli(
    consumer_key,
    consumer_secret,
    access_token_key,
    access_token_secret,
    delete_older_than,
    delete_everything_after,
    minimum_engagement,
    remove_favorites,
):
    """A simple program to delete all your tweets! Woohoo!"""
    from tweet_delete.deleter import Deleter
    from tweet_delete.util import td_format

    click.echo(click.style("🐦␡ starting tweet-delete".ljust(76) + "␡🐦", fg="green"))
    deleter = Deleter(
        consumer_key,
        consumer_secret,
        access_token_key,
        access_token_secret,
        delete_older_than,
        delete_everything_after,
        minimum_engagement,
        remove_favorites,
    )
    click.echo(click.style("🔑 validating credentials".ljust(77) + "🔑", fg="yellow"))
    creds = deleter.validate_creds()
    click.echo(
        highlight(
            json.dumps(creds.AsDict(), sort_keys=True, indent=2),
            JsonLexer(),
            TerminalFormatter(),
        )
    )
    click.echo(
        click.style(
            "👉 tweets older than {} will be deleted".format(
                td_format(delete_older_than.total_seconds())
            ).ljust(77)
            + "👈",
            fg="yellow",
        )
    )
    if delete_everything_after is not None:
        click.echo(
            click.style(
                "👉 only tweets created after {} will be deleted".format(
                    str(delete_everything_after)
                ).ljust(77)
                + "👈",
                fg="yellow",
            )
        )
    if remove_favorites:
        click.echo(
            click.style(
                "👉 favorites created after {} will be deleted".format(
                    str(delete_everything_after)
                ).ljust(77)
                + "👈",
                fg="yellow",
            )
        )
    else:
        click.echo(
            click.style(
                "👉 favorites will NOT be deleted".ljust(77) + "👈",
                fg="yellow",
            )
        )
    click.echo(click.style("🦅 off we go".ljust(77) + "🦅", fg="green"))
    deleter.run()



import twitter
import gevent
import click
import requests
from tweet_delete.util import td_format
from datetime import datetime
from dateutil import parser


class Deleter:
    def __init__(
        self,
        consumer_key,
        consumer_secret,
        access_token_key,
        access_token_secret,
        delete_older_than,
        delete_everything_after,
        minimum_engagement,
        remove_favorites,
    ):
        self.consumer_key = consumer_key
        self.consumer_secret = consumer_secret
        self.access_token_key = access_token_key
        self.access_token_secret = access_token_secret
        self.delete_older_than = delete_older_than
        self.delete_everything_after = delete_everything_after
        self.last_since_id = None
        self.minimum_engagement = minimum_engagement
        self.remove_favorites = remove_favorites
        self.ids_scheduled_for_deletion = set()

        self.api = self.get_api()

    def get_api(self):
        return twitter.Api(
            consumer_key=self.consumer_key,
            consumer_secret=self.consumer_secret,
            access_token_key=self.access_token_key,
            access_token_secret=self.access_token_secret,
            sleep_on_rate_limit=True,
            use_gzip_compression=True,
        )

    def validate_creds(self):
        return self.api.VerifyCredentials()

    def should_be_deleted_now(self, status):
        created_at = parser.parse(status.created_at).replace(tzinfo=None)
        expired_at = datetime.utcnow() - self.delete_older_than
        if self.delete_everything_after is not None:
            if created_at > self.delete_everything_after and created_at < expired_at:
                # The tweet was created after delete_everything_after and it has expired
                return True
        elif created_at < expired_at:
            # The tweet has expired
            return True
        return False

    def should_be_deleted(self, status):
        if self.delete_everything_after is None:
            return True
        created_at = parser.parse(status.created_at).replace(tzinfo=None)
        if created_at > self.delete_everything_after:
            # The tweet was created after delete_everything_after
            return True
        return False

    def schedule_delete(self, status):
        if (
            not self.should_be_deleted(status)
            or status.id in self.ids_scheduled_for_deletion
        ):
            return
        self.ids_scheduled_for_deletion.add(status.id)
        created_at = parser.parse(status.created_at).replace(tzinfo=None)
        expires_at = created_at + self.delete_older_than
        # include additional 5 second delay
        seconds_until = (expires_at - datetime.utcnow()).total_seconds() + 5
        seconds_until = max([10, seconds_until])
        gevent.spawn_later(seconds_until, self.check_delete, status)
        click.echo(
            click.style(
                "scheduled ID={} for future deletion in {}".format(
                    status.id, td_format(seconds_until)
                ),
                fg="blue",
            )
        )

    def check_delete(self, status):
        try:
            status_id = status.id
            click.echo(
                click.style(
                    "ID={} was scheduled for deletion, checking if it should be deleted".format(
                        status_id
                    ),
                    fg="cyan",
                )
            )
            # get a fresh API handle
            self.api = self.get_api()
            status = self.api.GetStatus(status_id)
            if not self.to_be_deleted(status):
                click.echo(
                    click.style("ID={} won't be deleted".format(status_id), fg="cyan")
                )
        except twitter.error.TwitterError as e:
            click.echo(click.style("caught exception: {}".format(e), fg="red"))

    def delete(self, status):
        click.echo(
            click.style(
                "🗑  deleting tweet ID={} favourites={} retweets={} text={}".format(
                    status.id, status.favorite_count, status.retweet_count, status.text
                ),
                fg="blue",
            )
        )
        self.api.DestroyStatus(status.id)
        if status.id in self.ids_scheduled_for_deletion:
            self.ids_scheduled_for_deletion.remove(status.id)

    def to_be_deleted(self, status):
        engagements = 2 * int(status.retweet_count) + int(status.favorite_count)
        if engagements < self.minimum_engagement:
            if self.should_be_deleted_now(status):
                self.delete(status)
                return True
            if self.should_be_deleted(status):
                self.schedule_delete(status)
                return True
        return False

    @staticmethod
    def zero_to_none(value):
        if int(value) == 0:
            return None
        else:
            return value

    @staticmethod
    def print_stats_for(name, values):
        from statistics import mean, harmonic_mean, median

        if not values or len(values) == 0:
            # skip because it's empty
            return

        click.echo(
            click.style(
                "🔢 {}: count={} min={} max={} mean={:.1f} harmonic_mean={:.1f} median={:.1f}".format(
                    name.ljust(16),
                    len(values),
                    min(values),
                    max(values),
                    mean(values),
                    harmonic_mean(values),
                    median(values),
                ),
                fg="magenta",
            )
        )

        # make a tiny histo
        from sparklines import sparklines
        import numpy as np

        hist, _ = np.histogram(values, bins=range(15))
        if hist.sum() > 0:
            hist = [Deleter.zero_to_none(v) for v in list(hist)]

            for line in sparklines(hist):
                click.echo(
                    click.style(
                        "📈 {}: {} {} {}".format(
                            (name + " histo").ljust(16), min(values), line, max(values)
                        ),
                        fg="magenta",
                    )
                )

    def check_for_tweets(self, last_max_id=None):
        statuses = [0]  # trick to force initial fetch
        max_id = None
        tweets_read = 0
        click.echo(
            click.style(
                "checking for tweets, starting from last_max_id={}".format(last_max_id),
                fg="cyan",
            )
        )
        # Read until either a) we run out of tweets or b) we start seeing the
        # same tweets as the previous run
        favourite_counts = []
        retweet_counts = []
        has_statuses = True
        while has_statuses:
            has_statuses = False
            statuses = self.api.GetUserTimeline(
                include_rts=True, exclude_replies=False, max_id=max_id, count=200
            )
            tweets_read += len(statuses)
            for status in statuses:
                has_statuses = True
                if max_id:
                    max_id = min([status.id, max_id])
                else:
                    max_id = status.id
                self.to_be_deleted(status)

                if not status.retweeted_status:
                    # ignore retweets when collecting stats
                    favourite_counts.append(status.favorite_count)
                    retweet_counts.append(status.retweet_count)

            # If the first tweet is too old to care about, stop fetching the
            # timeline
            if (
                self.delete_everything_after is not None
                and len(statuses) > 0
                and not self.should_be_deleted(statuses[0])
            ):
                break

        click.echo(
            click.style(
                "✅ done checking for tweets, tweets_read={} max_id={}".format(
                    tweets_read, max_id
                ),
                fg="cyan",
            )
        )

        if not last_max_id:
            Deleter.print_stats_for("favourites", favourite_counts)
            Deleter.print_stats_for("retweets", retweet_counts)
        return max_id

    def check_and_remove_favorites(self):
        if not self.remove_favorites:
            return None

        has_favourites = True
        max_id = None

        while has_favourites:
            has_favourites = False
            for fav in self.api.GetFavorites(count=200, max_id=max_id):
                has_favourites = True
                if max_id is None:
                    max_id = fav.id
                else:
                    max_id = min(max_id, fav.id)

                created_at = parser.parse(fav.created_at).replace(tzinfo=None)
                expires_at = created_at + self.delete_older_than
                if expires_at < datetime.utcnow():
                    click.echo(
                        click.style(
                            "deleting favorite with ID={}".format(fav.id),
                            fg="blue",
                        )
                    )
                    self.api.DestroyFavorite(status_id=fav.id)
                else:
                    click.echo(
                        click.style(
                            "favorite ID={} will be deleted later (after it expires)".format(
                                fav.id
                            ),
                            fg="cyan",
                        )
                    )

    def run(self):
        max_id = self.check_for_tweets()
        self.check_and_remove_favorites()
        gevent.sleep(60)
        delay = 5
        while True:
            try:
                # get a fresh API handle
                self.api = self.get_api()
                max_id = self.check_for_tweets(last_max_id=max_id)
                self.check_and_remove_favorites()
                gevent.sleep(3600)
                delay = 1
            except (
                twitter.error.TwitterError,
                requests.exceptions.RequestException,
            ) as e:
                delay = delay * 2.5
                delay = min([delay, 300])
                click.echo(click.style("caught exception: {}".format(e), fg="red"))
                click.echo(click.style("will retry in {}s".format(delay), fg="red"))
                gevent.sleep(delay)



def td_format(seconds):
    # Thanks https://stackoverflow.com/a/13756038
    seconds = int(seconds)
    periods = [
        ("year", 60 * 60 * 24 * 365),
        ("month", 60 * 60 * 24 * 30),
        ("day", 60 * 60 * 24),
        ("hour", 60 * 60),
        ("minute", 60),
        ("second", 1),
    ]

    strings = []
    for period_name, period_seconds in periods:
        if seconds >= period_seconds:
            period_value, seconds = divmod(seconds, period_seconds)
            has_s = "s" if period_value > 1 else ""
            strings.append("%s %s%s" % (period_value, period_name, has_s))

    return ", ".join(strings)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tweet_delete']

package_data = \
{'': ['*']}

install_requires = \
['Pygments>=2.6.1,<3.0.0',
 'click>=7.1.2,<8.0.0',
 'colorama>=0.4.3,<0.5.0',
 'gevent>=20.6.2,<21.0.0',
 'numpy>=1.19.1,<2.0.0',
 'python-dateutil>=2.8.1,<3.0.0',
 'python-twitter>=3.5,<4.0',
 'pytimeparse>=1.1.8,<2.0.0',
 'sparklines>=0.4.2,<0.5.0']

entry_points = \
{'console_scripts': ['tweet-delete = tweet_delete.entry:cli',
                     'vscode = vscode:vscode']}

setup_kwargs = {
    'name': 'tweet-delete',
    'version': '0.2.2',
    'description': 'Self-destructing Tweet tool',
    'long_description': '[![Build Status](https://travis-ci.org/brndnmtthws/tweet-delete.svg?branch=master)](https://travis-ci.org/brndnmtthws/tweet-delete) [![Maintainability](https://api.codeclimate.com/v1/badges/f50f5c31185dd44e5611/maintainability)](https://codeclimate.com/github/brndnmtthws/tweet-delete/maintainability) [![Test Coverage](https://api.codeclimate.com/v1/badges/f50f5c31185dd44e5611/test_coverage)](https://codeclimate.com/github/brndnmtthws/tweet-delete/test_coverage) [![PyPI version](https://badge.fury.io/py/tweet-delete.svg)](https://badge.fury.io/py/tweet-delete)\n\n# tweet-delete \U0001f99c🔫\n\n`tweet-delete` is a small Python tool for automatically deleting your tweets\nafter some specified amount of time. It is intended to be used to create\nself-destructing tweets. `tweet-delete` runs continuously, and will check\nyour timeline every hour to see if there are any new tweets which\nneed to be deleted. You may also specify a minimum engagement metric, which\nallows you to delete only the tweets that are junk 🗑.\n\nSelf-destructing tweets are the hippest, trendiest, coolest thing on\n[Twitter](https://twitter.com/) right now. Want to be cool and hip? You need\n`tweet-delete`. By creating artificial scarcity you can ten ex (10x) or\none-hundred ex (100x) your personal brand. 😎\n\nIn spite of the low technical barrier to entry for using this Twitter bot (or\nany similar ones), it does require following some instructions, and the\nTwitter dev account approval process is long and arduous. In other words, you\nwill easily be in the top 0.1% of technically skilled Twitter users. You will\nbe _super extra hip and cool_, and in the upper echelons of thought\nleadership, simply by using this tool. Wear your badge of honour loud and\nproud. Perhaps write "**These tweets self destruct.**" in your bio?\n\n## Quickstart\n\n_NOTE: This tool will delete your tweets. Please do not use this tool if you\ndon\'t want your tweets to be deleted._\n\n### 1. Set up Twitter Dev account\n\nTo get started, you\'ll need to go to\n[https://developer.twitter.com/en/apps](https://developer.twitter.com/en/apps)\nand set up a Twitter developer account, and create an "App".\n\nOnce you\'re approved (after several days or weeks of waiting), move on to the\nnext step.\n\n### 2. Generate API access tokens\n\n[Follow the instructions\nhere](https://developer.twitter.com/en/docs/basics/authentication/guides/access-tokens)\nto generate your API access tokens. Save these somewhere, as you\'ll be\nneeding them later.\n\n### 3. Find a place to run the codes\n\nYou\'ll need a computer somehere, perhaps somewhere up in the clouds, to run\nthe codes. For your convenience, this repo includes a [Helm\nchart](https://helm.sh/) to run this tool on Kubernetes, which is extremely\nAI these days (if you hadn\'t heard).\n\n### 4. Install\n\nThis is a standard Python package, which can be installed using pip:\n\n```ShellSession\n$ pip install tweet-delete\n...\n```\n\nAlternatively, you can simply use the [pre-built Docker\nimage](https://hub.docker.com/r/brndnmtthws/tweet-delete) if you prefer.\n\n### 5. Run\n\nRun the script by passing it the API keys you generated above. It will run\ncontinuously, and tweet all tweets that are older than `--delete-older-than`\ndays starting on Jan 1, 2019.\n\n```ShellSession\n$ tweet-delete \\\n    --consumer_key=<consumer_key> \\\n    --consumer_secret=<consumer_secret> \\\n    --access_token_key=<access_token_key> \\\n    --access_token_secret=<access_token_secret> \\\n    --delete_older_than="7 days" \\\n    --delete_everything_after=2019-01-01 \\\n    --minimum_engagement=1\n...\n```\n\nNow the script will run forever, and delete all of your tweets older than 7\ndays as long as it\'s running. Congratulations! 🎉🎊\U0001f973\n\n## Performance\n\nThe script features an asynchronous, event-driven core, base on the excellent\n[gevent](http://www.gevent.org/) library. `tweet-delete` should have no\ndifficulty achieving a tweet deletes per second (TDPS) throughput well in\nexcess of 1,000 TDPS. However, practically speaking, you will likely hit the\nTwitter API rate limits long before hitting the script\'s limits.\n\n## Deployment with Helm\n\nThere\'s a [Helm](https://helm.sh/) chart included for your convenience. To use the chart, copy [helm/tweet-delete/values.yaml](helm/tweet-delete/values.yaml) somewhere, and install the chart:\n\nNow install the chart:\n\n```ShellSession\n$ cp helm/tweet-delete/values.yaml myvalues\n$ helm upgrade --install tweet-delete helm/tweet-delete -f myvalues.yaml\nRelease "tweet-delete" has been upgraded. Happy Helming!\nLAST DEPLOYED: Wed Mar 13 15:08:31 2019\nNAMESPACE: default\nSTATUS: DEPLOYED\n\nRESOURCES:\n==> v1/Deployment\nNAME          READY  UP-TO-DATE  AVAILABLE  AGE\ntweet-delete  0/1    1           0          46s\n\n==> v1/Pod(related)\nNAME                           READY  STATUS             RESTARTS  AGE\ntweet-delete-79bdbd995b-2mrmj  0/1    ContainerCreating  0         0s\n```\n\nSweeeeeet 😎\n\n## How can I recover deleted tweets?\n\nYou can\'t! They\'re gone!\n\nIf your account is public, it\'s possible that your tweets have been archived\nsomewhere. The internet is a semi-free and open place, so it\'s relatively\neasy to archive anything you find on it. For example, you may want to try\nrecovering your old tweets from\n[https://snapbird.org/](https://snapbird.org/).\n\n## Limitations\n\nTwitter does not let you retrieve more than 3,200 tweets from their public\nAPI, thus you cannot delete more than 3,200.\n',
    'author': 'Brenden Matthews',
    'author_email': 'brenden@brndn.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/brndnmtthws/tweet-delete',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
